local Class = require("lib.Class")


local Events = Class:derive("Events")

function Events:new(eventMustExist)
   self.handlers = {}
   self.eventMustExist = (eventMustExist == nil) or eventMustExist
end

local function indexOf(evtTbl, callback)
   if evtTbl == nil or callback == nil then
      return -1
   end

   for i = 1, #evtTbl do
      if evtTbl[i] == callback  then
	 return i
      end
   end
   
   return -1
end


function Events:add(evtType)
   assert(self.handlers[evtType] == nil, "HATA: Event " .. evtType .. " already exists!")
   self.handlers[evtType] = {}
end

function Events:remove(evtType)
   self.handlers[evtType] = nil
end

function Events:hook(evtType, callback)
   assert(type(callback) == "function", "HATA: callback parameter must be a function!")

   if self.eventMustExist then
      assert(self.handlers[evtType] ~= nil, "HATA: Event of type " .. evtType .. " does not exists!")
   elseif self.handlers[evtType] == nil then
      self:add(evtType)
   end

--   if indexOf(self.handlers[evtType], callback) > -1 then end

   assert(indexOf(self.handlers[evtType], callback) == -1, "HATA: callback has already been registered!")
   
   local tbl = self.handlers[evtType]
   tbl[#tbl + 1] = callback
end

function Events:unhook(evtType, callback)
   assert(type(callback) == "function", "HATA: callback parameter must be a function!")
   if self.handlers[evtType] == nil then return end
   local index = indexOf(self.handlers[evtType], callback)
   if index > -1 then
      table.remove(self.handlers[evtType], index)
   end
end

function Events:clear(evtType)
   if evtType == nil then
      for k, v in pairs(self.handlers) do
	 self.handlers[k] = {}
      end
   elseif self.handlers[evtType] ~= nil then
      self.handlers[evtType] = {}
   end
end


function Events:invoke(evtType, ...)
   if self.handlers[evtType] == nil then return end

--   assert(self.handlers[evtType] ~= nil, "HATA: Event of type " .. evtType .. " does not exists!")

   local tbl = self.handlers[evtType]
   for i = 1, #tbl do
      tbl[i](...)
   end

end


return Events
